var map;
var FloorToRecords = {};
var currentFloor = 1;
var floorCount = 3;
var resizeTimer = null;

function onLoad() {
    layout();
    window.onresize = function() {
        if (resizeTimer == null) {
            resizeTimer = window.setTimeout(function() {
                resizeTimer = null;
                layout();
            }, 300);
        }
    };
    document.body.onclick = dismissSearchResultsDiv;

    if (!GBrowserIsCompatible()) {
        alert("Sorry, the Google Maps API is not compatible with this browser");
        return;
    }
    
    /*
     *  Preparing the data
     */
    for (var i = 0; i < floorCount; i++) {
        FloorToRecords[i] = [];
    }
    for (var id in Records) {
        var record = Records[id];
        if (record != null) {
            if (typeof record.floor == "number") {
                FloorToRecords[record.floor].push(record);
            } else {
                for (var x = 0; x < record.floor.length; x++) {
                    FloorToRecords[record.floor[x]].push(record);
                }
            }
        }
    }
    
    /*
     *  Creating the map and UI
     */
    var div = document.getElementById("map");

    map =  MITFloorPlans.create(div, MITBuildings["W92"], [ currentFloor ], { /* draggableCursor: "crosshair" */ });
    map.addControl(new GLargeMapControl());
    map.addControl(new GOverviewMapControl());
    map.addControl(new GScaleControl());
    map.enableContinuousZoom();
    map.enableDoubleClickZoom();
    
    GEvent.addListener(map, "click", onMapClick);
    
    createControls(currentFloor);
    document.getElementById("search-textbox").onkeypress = onSearchKeyPress;
}

function getWindowDimensions() {
    var width = 0, height = 0;
    if (typeof( window.innerWidth ) == 'number') {
        //Non-IE
        width = window.innerWidth;
        height = window.innerHeight;
    } else if (document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight)) {
        //IE 6+ in 'standards compliant mode'
        width = document.documentElement.clientWidth;
        height = document.documentElement.clientHeight;
    } else if (document.body && (document.body.clientWidth || document.body.clientHeight)) {
        //IE 4 compatible
        width = document.body.clientWidth;
        height = document.body.clientHeight;
    }
    return { width: width, height: height };
}

function layout() {
    var margin = 10; // pixels
    var topHeight = 70;
    var leftWidth = 100;
    
    var d = getWindowDimensions();
    
    var topPanel = document.getElementById("top-panel");
    var leftPanel = document.getElementById("left-panel");
    var rightPanel = document.getElementById("right-panel");
    
    topPanel.style.top = margin + "px";
    topPanel.style.left = margin + "px";
    topPanel.style.width = (d.width - 2 * margin) + "px";
    topPanel.style.height = topHeight + "px";
    
    leftPanel.style.top = (topHeight + 2 * margin) + "px";
    leftPanel.style.left = margin + "px";
    leftPanel.style.width = leftWidth + "px";
    leftPanel.style.height = (d.height - topHeight - 3 * margin) + "px";
    
    rightPanel.style.top = (topHeight + 2 * margin) + "px";
    rightPanel.style.left = (leftWidth + 2 * margin) + "px";
    rightPanel.style.width = (d.width - leftWidth - 3 * margin) + "px";
    rightPanel.style.height = (d.height - topHeight - 3 * margin) + "px";
}

function createControls(initialFloor) {
    var container = document.getElementById("floor-radio-buttons");
    var html = [];
    for (var f = floorCount - 1; f >= 0; f--) {
        html.push(
            "<a class='" + (f == currentFloor ? "floor-link-selected" : "floor-link") + "' href='javascript:setFloor(" + f + ");' id='floor-link-" + f + "'>" +
                "Floor " + f +
            "</a>"
        );
    }
    
    container.innerHTML = html.join("");
    
    recreateMarkers(initialFloor);
}

function setFloor(f, cont) {
    if (f != currentFloor) {
        var floorIndicator = document.getElementById("floor-indicator");
        floorIndicator.innerHTML = "<span>floor " + f + "</span>";
        floorIndicator.style.display = "block";
        
        document.getElementById("floor-link-" + currentFloor).className = "floor-link";
        document.getElementById("floor-link-" + f).className = "floor-link-selected";
        
        window.setTimeout(function() {
            removeNewPoint();
            
            currentFloor = f;
            map.showFloors([ f ]);
            recreateMarkers(f);
            
            window.setTimeout(function() {
                floorIndicator.style.display = "none";
            }, 500);
            
            if (cont) {
                cont();
            }
        }, 10);
    }
}

function recreateMarkers(floor) {
    map.clearOverlays();
    
    var records = FloorToRecords[floor];
    for (var i = 0; i < records.length; i++) {
        renderRecord(records[i]);
    }
}

function renderRecord(record, open) {
    var icon = null;
    /*if (record.image != "") {
        icon = createImageIcon(record.image);
    } else*/ {
        var f = record.functions.length > 0 ? record.functions[0] : "";
        switch (f) {
        case "men's room":
            icon = createGoogleMyMapsMarker("man");
            break;
        case "ladies' room":
            icon = createGoogleMyMapsMarker("woman");
            break;
        case "kitchen":
            icon = createGoogleMyMapsMarker("restaurant");
            break;
        case "cafeteria":
            icon = createGoogleMyMapsMarker("snack_bar");
            break;
        case "supplies room":
            icon = createGoogleMyMapsMarker("mechanic");
            break;
        case "printer":
            icon = createGoogleMyMapsMarker("printer");
            break;
        case "photocopier":
            icon = createGoogleMyMapsMarker("photocopier");
            break;
        case "conference room":
            icon = createGoogleMyMapsMarker("conference");
            break;
        case "person":
            icon = createGoogleMyMapsMarker("person");
            break;
        case "elevators":
            icon = createGoogleMyMapsMarker("elevators");
            break;
        default:
            icon = new GIcon(G_DEFAULT_ICON);
        }
    }
    
    var latlng = new GLatLng(record.lat, record.lng);
    var marker = new GMarker(latlng, { icon: icon });
    var onclick = function() { onMarkerClick(record, marker); };
    
    GEvent.addListener(marker, "click", onclick);
    map.addOverlay(marker);
    record.marker = marker;
    
    if (open) {
        onclick();
    }
}

function onMarkerClick(record, marker) {
    var html = 
        "<div class='record-bubble'>" +
            "<div class='record-bubble-functions'>" +
                record.functions.join(", ") +
            "</div>";
            
    if (record.label != "") {
        html +=
            "<div class='record-bubble-label'>" + record.label + "</div>";
    }
    if (record.image != "") {
        html +=
            "<div class='record-bubble-image'><img src='" + record.image + "' /></div>";
    }
    if (record.description != "") {
        html +=
            "<div class='record-bubble-description'>" + record.description + "</div>";
    }
    if ("id" in record) {
        html +=
                "<div class='record-bubble-controls'>" +
                    "<button onclick=\"removeRecord('" + record.id + "')\">Remove</button>" +
                "</div>";
    }
            
    html +=
        "</div>";
        
    marker.openInfoWindow(html);
}

var clickMode = "add-new-point";
function onMapClick(overlay, latlng) {
    /*if (latlng) {
        switch (clickMode) {
        case "add-new-point":
            addNewPoint(latlng);
            break;
        }
    }*/
}

var newPoint = null;
function addNewPoint(clickedLatlng) {
    removeNewPoint();
    
    var id = "p" + new Date().getTime() + "-" + Math.floor(Math.random() * 1000000);
    var latlng = clickedLatlng || map.getCenter();
    
    var marker = new GMarker(latlng, { draggable: true, icon: createNewMarker() });
    GEvent.addListener(marker, "click", openNewPointInfo);
    GEvent.addListener(marker, "dragstart", function() {
        map.closeInfoWindow();
    });
    GEvent.addListener(marker, "dragend", function() {
        var latlng = marker.getLatLng();
        newPoint.lat = latlng.lat();
        newPoint.lng = latlng.lng();
        openNewPointInfo();
    });
    map.addOverlay(marker);
    
    newPoint = {
        marker:         marker, 
        id:             id,
        lat:            latlng.lat(),
        lng:            latlng.lng(),
        floor:          currentFloor,
        label:          "",
        description:    "",
        functions:      []
    };
    
    openNewPointInfo();
}

function removeNewPoint() {
    if (newPoint != null) {
        map.removeOverlay(newPoint.marker);
        newPoint = null;
    }
}

function openNewPointInfo() {
    var functionGroups = [
        [
            "person",
            "office",
            "work desks"
        ],
        [
            "conference room",
            "lecture room",
            "lounge"
        ],
        [
            "printer",
            "photocopier",
            "kitchen"
        ],
        [
            "men's room",
            "ladies' room"
        ]
   ];
    var html = 
        "<div class='new-point-bubble'>" +
            "<div class='new-point-form'>" +
                "<div class='new-point-form-section'>" +
                    "<div class='new-point-form-section-heading'>What is here?</div>" +
                    "<table cellspacing='5'><tr valign='top'>";
            
    for (var g = 0; g < functionGroups.length; g++) {
        var functionGroup = functionGroups[g];
        
        html += "<td>";
        for (var i = 0; i < functionGroup.length; i++) {
            html += "<input type='radio' name='functions' value=\"" + functionGroup[i] + "\" />" + functionGroup[i] + "<br/>";
        }
        html += "</td>";
    }
    html += 
                    "</tr></table>" +
                    "<div><input type='radio' name='functions' value='other' />other: <input type='text' id='other-function' /></div>" +
                "</div>" +
                "<div class='new-point-form-section'>" +
                    "<div class='new-point-form-section-heading'>Name (optional) <span class='new-point-form-example'>(e.g., G531, Cafeteria, John Doe)</span></div>" +
                    "<input type='text' id='name-textbox' style='width: 100%;'></input>" +
                "</div>" +
                "<div class='new-point-form-section'>" +
                    "<div class='new-point-form-section-heading'>Image's URL (optional) <span class='new-point-form-example'>(e.g., a person's portrait)</span></div>" +
                    "<input type='text' id='image-textbox' style='width: 100%;'></input>" +
                "</div>" +
                "<div class='new-point-form-section'>" +
                    "<div class='new-point-form-section-heading'>Description (optional)</div>" +
                    "<textarea id='description-textarea' style='width: 100%;'></textarea>" +
                "</div>" +
            "</div>" +
            "<div class='new-point-commit-controls'>" +
                "<table width='100%' cellspacing='5' cellpadding='0'><tr valign='top'>" +
                    "<td width='5%'>" +
                        "<button onclick='removeNewPoint()'>Cancel</button>" +
                    "</td>" +
                    "<td align='center'>Drag this marker to re-position it!</td>" +
                    "<td width='5%' align='right'>" +
                        "<button onclick='saveNewPoint()'>Save Point</button>" +
                    "</td>" +
                "</tr></table>" +
            "</div>" +
        "</div>";
        
    newPoint.marker.openInfoWindow(html);
}

function saveNewPoint() {
    newPoint.label = document.getElementById("name-textbox").value;
    newPoint.description = document.getElementById("description-textarea").value;
    newPoint.image = document.getElementById("image-textbox").value;
    
    var params = [
        "id=" + encodeUrlString(newPoint.id),
        "lat=" + encodeURIComponent(newPoint.lat),
        "lng=" + encodeURIComponent(newPoint.lng),
        "floor=" + encodeURIComponent(newPoint.floor),
        "label=" + encodeUrlString(newPoint.label),
        "description=" + encodeUrlString(newPoint.description),
        "image=" + encodeUrlString(newPoint.image)
    ];
    var functions = document.getElementsByName("functions");
    var fCount = 0;
    for (var i = 0; i < functions.length; i++) {
        var f = functions[i];
        if (f.checked) {
            var value = (f.value != "other") ? f.value : document.getElementById("other-function").value;
            if (value.length > 0) {
                newPoint.functions.push(value);
                params.push("function" + (fCount++) + "=" + encodeUrlString(value));
            }
        }
    }
    
    SimileAjax.XmlHttp.get(
        "app/add.php?" + params.join("&"),
        function() {
            alert("Sorry, failed to add the new point!");
        },
        function() {
            map.removeOverlay(newPoint.marker);
            delete newPoint.marker;
            
            Records[newPoint.id] = newPoint;
            FloorToRecords[newPoint.floor].push(newPoint);
            
            renderRecord(newPoint, true);
            
            newPoint = null;
        }
    );
}

function removeRecord(id) {
    SimileAjax.XmlHttp.get(
        "app/remove.php?id=" + encodeURIComponent(id),
        function() {
            alert("Sorry, failed to remove the point!");
        },
        function() {
            var record = Records[id];
            var records = FloorToRecords[record.floor];
            for (var i = 0; i < records.length; i++) {
                if (record == records[i]) {
                    records.splice(i, 1);
                    break;
                }
            }
            delete Records[id];
            
            recreateMarkers(currentFloor);
        }
    );
}

function onSearchKeyPress(evt) {
    evt = (evt) ? evt : event;
    if (evt.keyCode != 13) { // enter
        dismissSearchResultsDiv();
        return;
    }
    
    var searchTextbox = document.getElementById("search-textbox");
    var s = searchTextbox.value.toLowerCase().trim();
    if (s.length == 0) {
        return;
    }
    
    var results = [];
    for (var id in Records) {
        var record = Records[id];
        if (record != null) {
            var found = false;
            for (var f = 0; f < record.functions.length; f++) {
                if (record.functions[f].toLowerCase().indexOf(s) >= 0) {
                    found = true;
                    break;
                }
            }
            
            found = found || (record.label.toLowerCase().indexOf(s) >= 0);
            found = found || (record.description.toLowerCase().indexOf(s) >= 0);
            
            if (found) {
                results.push(record);
            }
        }
    }
    
    if (results.length == 1) {
        goToRecord(results[0]);    
    } else if (results.length > 1) {
        var coords = SimileAjax.DOM.getPageCoordinates(searchTextbox);
        
        var searchResultsDiv = document.getElementById("search-results");
        searchResultsDiv.innerHTML = "";
        searchResultsDiv.style.top = (coords.top + searchTextbox.offsetHeight) + "px";
        searchResultsDiv.style.right = (getWindowDimensions().width - coords.left - searchTextbox.offsetWidth) + "px";
        
        var createLink = function(record) {
            var a = document.createElement("a");
            var f = (typeof record.floor == "number") ? ("floor " + record.floor) : ("floors " + record.floor.join(", "));
            
            a.className = "search-result";
            a.href = "javascript:{}";
            a.onclick = function() { dismissSearchResultsDiv(); goToRecord(record); };
            a.innerHTML = record.functions.join(", ") + " (" + f + "): " + record.label;
            searchResultsDiv.appendChild(a);
        };
        
        for (var i = 0; i < results.length && i < 10; i++) {
            createLink(results[i]);
        }
        
        searchResultsDiv.style.display = "block";
    }
}

function dismissSearchResultsDiv() {
    var searchResultsDiv = document.getElementById("search-results");
    searchResultsDiv.style.display = "none";
}

function goToRecord(record) {
    var floor;
    if (typeof record.floor == "number") {
        floor = record.floor;
    } else {
        floor = record.floor[0];
        
        // priority for the current floor
        for (var x = 0; x < record.floor.length; x++) {
            if (record.floor[x] == currentFloor) {
                floor = currentFloor;
                break;
            }
        }
    }
    
    var panning = function() {
        map.panTo(new GLatLng(record.lat, record.lng));
        if ("marker" in record) {
            onMarkerClick(record, record.marker);
        }
    };
    
    if (floor != currentFloor) {
        setFloor(floor, panning);
    } else {
        panning();
    }
}

function encodeUrlString(s) {
    return encodeURIComponent(encodeJavascriptString(s));
}

function encodeUrlStringArray(a) {
    var b = [];
    for (var i = 0; i < a.length; i++) {
        b.push("'" + encodeJavascriptString(a[i]) + "'");
    }
    return encodeURIComponent('[' + b.join(",") + ']');
}

function encodeJavascriptString(s) {
    return s.replace(/\n/g, "\\n").replace(/\r/g, "\\r");
}
